import base64
import random

from passlib.hash import atlassian_pbkdf2_sha1


def generate_client_token():
    # Token format : length 33
    # - 12 randoms int -> TokenID
    # a semi-colon ":"
    # - 20 random bytes -> secret
    token = ""
    for i in range(12):
        token += str(random.randint(0, 9))
    token += ":"
    for i in range(20):
        token += chr(random.randint(0, 255))

    token_bytes = bytearray()
    for i in range(len(token)):
        token_bytes.append(ord(token[i]))

    return base64.b64encode(token_bytes)


def hash_token(token_bytearray, salt=""):
    raw_token = base64.b64decode(token_bytearray)
    raw_token_secret = raw_token[13:]
    encoded_token_secret = base64.b64encode(raw_token_secret)

    if salt:
        a = atlassian_pbkdf2_sha1.using(salt=salt)
    else:
        a = atlassian_pbkdf2_sha1.using()

    hashed_token = a.hash(encoded_token_secret)
    return hashed_token


def extract_token_id(token_bytearray):
    raw_token = base64.b64decode(token_bytearray)
    return raw_token[:12]


def extract_token_secret(token_bytearray):
    raw_token = base64.b64decode(token_bytearray)
    raw_token_secret = raw_token[13:]
    return base64.b64encode(raw_token_secret)


def generate_confluence_token():
    client_token = generate_client_token()
    print(f"[+] API token : {client_token.decode()}")
    print(f"\t[*] Token ID         : {extract_token_id(client_token).decode()}")
    print(f"\t[*] Token Secret b64 : {extract_token_secret(client_token).decode()}")
    hashed_token = hash_token(client_token)
    print(f"[+] Hashed token secret : {hashed_token}")


generate_confluence_token()
